using System;
using System.IO;
using System.Configuration;
using System.Web;

using Db4objects.Db4o;
using Db4objects.Db4o.Config;
using System.Collections.Generic;

namespace Deerchao.Db4oWebSupport
{
    public class Db4oHttpModule : IHttpModule
    {
        public void Init(HttpApplication application)
        {
            application.EndRequest += new EventHandler(Application_EndRequest);
        }

        private void Application_EndRequest(object sender, EventArgs e)
        {
            HttpApplication application = (HttpApplication)sender;
            HttpContext context = application.Context;

            Dictionary<string, IObjectContainer> clients = context.Items[clientsCacheKey] as Dictionary<string, IObjectContainer>;

            if (clients != null)
            {
                foreach (IObjectContainer client in clients.Values)
                    client.Close();

                clients.Clear();
            }
        }

        static readonly string clientsCacheKey = "db4o-clients-in-httpcontext";

        public static IObjectContainer GetClient(string connectionName)
        {
            HttpContext context = HttpContext.Current;

            Dictionary<string, IObjectContainer> clients = context.Items[clientsCacheKey] as Dictionary<string, IObjectContainer>;

            if (clients == null)
            {
                clients = new Dictionary<string, IObjectContainer>();
                context.Items[clientsCacheKey] = clients;
            }

            if (!clients.ContainsKey(connectionName))
            {
                clients.Add(connectionName, GetServer(connectionName).OpenClient());
            }

            return clients[connectionName];
        }

        private static string GetClientCacheKey(string connectionName)
        {
            return string.Format("db4o-client-in-httpcontext-{0}", connectionName);
        }

        public void Dispose()
        {
        }

        private static Dictionary<string, IObjectServer> servers = new Dictionary<string, IObjectServer>();

        private static IObjectServer GetServer(string connectionName)
        {
            if (!servers.ContainsKey(connectionName))
            {
                string connectionString = ConfigurationManager.ConnectionStrings[connectionName].ConnectionString;
                string filePath = HttpContext.Current.Server.MapPath(connectionString);
                servers.Add(connectionName, Db4oFactory.OpenServer(filePath, 0));
            }
            return servers[connectionName];
        }

        public static void CloseServers()
        {
            if (servers != null)
            {
                foreach (IObjectServer server in servers.Values)
                {
                    if (server != null)
                        server.Close();
                }
                servers.Clear();
            }
        }
    }
}

