using System;
using System.Collections.Generic;
using System.Web.Security;
using Db4objects.Db4o;
using Db4objects.Db4o.Query;
using Db4objects.Db4o.Ext;

namespace Deerchao.Db4oWebSupport.Security
{
    public class db4oMembershipProviderManager
    {
        private static db4oMembershipProviderManager instance;
        // Lock synchronization object
        private static object syncLock = new object();
        private string _connStringName;

        protected db4oMembershipProviderManager()
        {

        }

        public static db4oMembershipProviderManager GetManager(string connStringName)
        {
            if (instance == null)
            {
                lock (syncLock)
                {
                    if (instance == null)
                    {
                        instance = new db4oMembershipProviderManager();
                        instance._connStringName = connStringName;
                    }
                }
            }

            return instance;
        }

        private IObjectContainer Container
        {
            get
            {
                return Db4oHttpModule.GetClient(this._connStringName);
            }
        }

        #region relative to db4oApplication

        public db4oApplication GetApplicationByName(string name)
        {
            IQuery qr = Container.Query();
            qr.Constrain(typeof(db4oApplication));
            qr.Descend("_applicationName").Constrain(name);
            IObjectSet result = qr.Execute();
            if (result.Count == 0)
            {
                db4oApplication newApplication = new db4oApplication();
                newApplication.ApplicationName = name;
                Container.Set(newApplication);
                return newApplication;
            }
            else
                return (db4oApplication)result.Next();
        }

        public Db4oUUID GetApplicationUUID(string name)
        {
            return Container.Ext().GetObjectInfo(GetApplicationByName(name)).GetUUID();
        }

        public db4oApplication GetApplicationByUUID(Db4oUUID uuid)
        {
            return (db4oApplication)Container.Ext().GetByUUID(uuid);
        }
        #endregion

        #region relative to db4oMembershipUser
        public db4oMembershipUser GetUserByEmail(string email)
        {
            IQuery query = Container.Query();
            query.Constrain(typeof(db4oMembershipUser));
            query.Descend("_email").Constrain(email);
            IObjectSet result = query.Execute();
            if (result.HasNext())
                return (db4oMembershipUser)result.Next();
            else
                return null;
        }

        public db4oMembershipUser GetUserByUserName(string username)
        {
            IQuery query = Container.Query();
            query.Constrain(typeof(db4oMembershipUser));
            query.Descend("_userName").Constrain(username);
            IObjectSet result = query.Execute();
            if (result.HasNext())
                return (db4oMembershipUser)result.Next();
            else
                return null;
        }

        public db4oMembershipUser GetUserById(string userId)
        {
            IQuery query = Container.Query();
            query.Constrain(typeof(db4oMembershipUser));
            query.Descend("_id").Constrain(userId);
            IObjectSet result = query.Execute();
            if (result.HasNext())
                return (db4oMembershipUser)result.Next();
            else
                return null;
        }

        public IList<db4oMembershipUser> GetUsersByName(string usernameToMatch)
        {
            return Container.Query<db4oMembershipUser>(
                delegate(db4oMembershipUser candidate)
                {
                    bool canReturn = candidate.UserName.Contains(usernameToMatch);
                    return canReturn;
                });
        }

        /// <summary>
        /// Return all users
        /// </summary>
        /// <param name="usernameToMatch"></param>
        /// <returns></returns>
        public IList<db4oMembershipUser> GetAllUsers()
        {
            List<db4oMembershipUser> users = new List<db4oMembershipUser>();
            foreach (db4oMembershipUser user in Container.Query<db4oMembershipUser>(typeof(db4oMembershipUser)))
            {
                users.Add(user);
            }

            return users;
        }

        public MembershipUser CreateUser(string username, string email, MembershipPasswordFormat passwordFormat, string passwordAnswer, bool isApproved, string salt, string pass, string applicationName, string providerName)
        {
            db4oMembershipUser newUser = new db4oMembershipUser();
            newUser.Id = Guid.NewGuid().ToString();
            newUser.CreateDate = DateTime.Now;
            newUser.IsApproved = isApproved;
            newUser.Password = pass;
            newUser.PasswordAnswer = passwordAnswer;
            newUser.PasswordFormat = (int)passwordFormat;
            newUser.PasswordSalt = salt;
            newUser.UserName = username;
            newUser.Email = email;
            Container.Set(newUser);

            if (Container.Ext().IsStored(newUser))
            {
                db4oMembershipActivity registerCreate = new db4oMembershipActivity();
                registerCreate.Membership = newUser;
                Container.Set(registerCreate);
                return GetMembershipBydb4oObject(newUser, GetApplicationByName(applicationName), providerName);
            }
            else
                return null;
        }

        public MembershipUser GetMembershipBydb4oObject(db4oMembershipUser user, db4oApplication application, string providerName)
        {
            DateTime lastLoginDate = DateTime.MinValue;
            DateTime lastActivityDate = DateTime.MinValue;
            IQuery qr = Container.Query();
            //get lastactivity date

            qr.Constrain(typeof(db4oMembershipActivity));
            qr.Descend("_membership").Constrain(user);
            qr.Descend("_application").Constrain(application);
            qr.Descend("_lastActivityDate").OrderDescending();
            IObjectSet result = qr.Execute();
            db4oMembershipActivity lastAct = new db4oMembershipActivity();
            if (result.Count > 0)
            {
                lastAct = ((db4oMembershipActivity)result.Next());
                lastActivityDate = lastAct.LastActivityDate;
            }
            //get lastlogin date.. if last activity is loggin, don't need query again
            if (lastAct.ActivityType == ActivityType.Logging)
            {
                lastLoginDate = lastAct.LastActivityDate;
            }
            else
            {
                qr = Container.Query();
                qr.Constrain(typeof(db4oMembershipActivity));
                qr.Descend("_membership").Constrain(user);
                qr.Descend("_application").Constrain(application);
                qr.Descend("_activityType").Constrain(ActivityType.Logging);
                qr.Descend("_lastActivityDate").OrderDescending();
                result = qr.Execute();
                if (result.Count > 0)
                    lastLoginDate = ((db4oMembershipActivity)result.Next()).LastActivityDate;
            }

            return new MembershipUser(providerName, user.UserName, user.Id, user.Email, user.PasswordQuestion, user.Comment, user.IsApproved, user.IsLoockedOut, user.CreateDate, lastLoginDate, lastActivityDate, user.LastPasswordChangedDate, user.LastLockOutDate);
        }

        public void UpdateUser(db4oMembershipUser templateUser)
        {
            db4oMembershipUser update = this.GetUserById(templateUser.Id);
            update.Comment = templateUser.Comment;
            update.Email = templateUser.Email;
            update.IsApproved = templateUser.IsApproved;
            update.IsLoockedOut = templateUser.IsLoockedOut;
            update.PasswordQuestion = templateUser.PasswordQuestion;
            update.UserName = templateUser.UserName;
            Container.Set(update);
        }

        public bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            db4oMembershipUser user = GetUserByUserName(username);
            if (user != null)
            {
                if (deleteAllRelatedData)
                {
                    //TODO: delete roles, profiles, etc
                    IQuery qr = Container.Query();
                    qr.Constrain(typeof(db4oMembershipActivity));
                    qr.Descend("_membership").Constrain(user);
                    foreach (db4oMembershipActivity activity in qr.Execute())
                    {
                        Container.Delete(activity);
                    }
                }
                Container.Delete(user);
                Container.Commit();
                return true;
            }
            return false;
        }

        public bool ChangePassword(string userName, string encPass, string salt, MembershipPasswordFormat _passwordFormat)
        {
            db4oMembershipUser user = GetUserByUserName(userName);
            if (user != null)
            {
                user.PasswordFormat = (int)_passwordFormat;
                user.Password = encPass;
                user.PasswordSalt = salt;
                user.LastPasswordChangedDate = DateTime.Now;
                Container.Set(user);
                return true;
            }
            else
            {
                return false;
            }
        }

        public bool ChangePasswordQuestionAndAnswer(string userName, string newPasswordQuestion, string newPasswordAnswer)
        {
            db4oMembershipUser user = GetUserByUserName(userName);
            if (user != null)
            {
                user.PasswordQuestion = newPasswordQuestion;
                user.PasswordAnswer = newPasswordAnswer;
                Container.Set(user);
                return true;
            }
            else
            {
                return false;
            }
        }

        public MembershipUser GetMembershipByNames(string username, string applicationName, string providerName)
        {
            return GetMembershipBydb4oObject(GetUserByUserName(username), GetApplicationByName(applicationName), providerName);
        }

        public MembershipUser GetMembershipByUserId(string userId, string applicationName, string providerName)
        {
            return GetMembershipBydb4oObject(GetUserById(userId), GetApplicationByName(applicationName), providerName);
        }

        #endregion

        #region relative to db4oMembershipActivity
        public void RegisterActivity(string username, string applicationName, ActivityType type)
        {
            db4oMembershipActivity act = new db4oMembershipActivity();
            act.Application = GetApplicationByName(applicationName);
            act.Membership = GetUserByUserName(username);
            act.ActivityType = type;
            Container.Set(act);
        }
        #endregion
    }
}
