using System;
using System.Collections.Generic;
using System.Web.Security;
using System.Configuration.Provider;
using Db4objects.Db4o;
using Db4objects.Db4o.Query;

namespace Deerchao.Db4oWebSupport.Security
{
    public class db4oRoleProviderManager
    {
        private static db4oRoleProviderManager instance;
        // Lock synchronization object
        private static object syncLock = new object();
        private string _connStringName;

        protected db4oRoleProviderManager()
        {

        }

        public static db4oRoleProviderManager GetManager(string connStringName)
        {
            if (instance == null)
            {
                lock (syncLock)
                {
                    if (instance == null)
                    {
                        instance = new db4oRoleProviderManager();
                        instance._connStringName = connStringName;
                    }
                }
            }

            return instance;
        }

        private IObjectContainer Container
        {
            get
            {
                return Db4oHttpModule.GetClient(this._connStringName);
            }
        }

        #region relative to db4oApplication

        public db4oApplication GetApplicationByName(string name)
        {
            IQuery qr = Container.Query();
            qr.Constrain(typeof(db4oApplication));
            qr.Descend("_applicationName").Constrain(name);
            IObjectSet result = qr.Execute();
            if (result.Count == 0)
            {
                db4oApplication newApplication = new db4oApplication();
                newApplication.ApplicationName = name;
                Container.Set(newApplication);
                return newApplication;
            }
            else
                return (db4oApplication)result.Next();
        }

        #endregion

        public db4oRole GetRoleByName(string roleName, string applicationName)
        {
            IQuery qr = Container.Query();
            qr.Constrain(typeof(db4oRole));
            qr.Descend("_rolename").Constrain(roleName);
            qr.Descend("_application").Constrain(GetApplicationByName(applicationName));
            IObjectSet result = qr.Execute();
            if (result.HasNext())
                return (db4oRole)result.Next();
            else
                return null;
        }

        public db4oRole CreateRole(string roleName, string ApplicationName)
        {
            db4oRole role = new db4oRole();
            role.Application = GetApplicationByName(ApplicationName);
            role.RoleName = roleName;
            Container.Set(role);
            if (Container.Ext().IsStored(role))
                return role;
            else
                return null;
        }

        public string[] GetUsersInRole(string roleName, string applicationName)
        {
            db4oRole role = GetRoleByName(roleName, applicationName);
            if (role != null)
            {
                IQuery qr = Container.Query();
                qr.Constrain(typeof(db4oUserInRole));
                qr.Descend("_role").Constrain(role);
                IObjectSet result = qr.Execute();
                string[] ret = new string[result.Count];
                int i = 0;
                while (result.HasNext())
                {
                    ret[i] = ((db4oUserInRole)result.Next()).UserName;
                    i++;
                }
                return ret;
            }
            else
                throw new ArgumentException("Role Name not found");
        }

        public bool DeleteRole(string roleName, string applicationName)
        {
            db4oRole role = GetRoleByName(roleName, applicationName);
            Container.Delete(role);
            Container.Commit();
            return !(Container.Ext().IsStored(role));
        }

        public string[] GetAllRoles(string applicationName)
        {
            IQuery qr = Container.Query();
            qr.Constrain(typeof(db4oRole));
            qr.Descend("_application").Constrain(GetApplicationByName(applicationName));
            IObjectSet result = qr.Execute();
            string[] ret = new string[result.Count];
            int i = 0;
            while (result.HasNext())
            {
                db4oRole role = (db4oRole)result.Next();
                ret[i] = role.RoleName;
                i++;
            }
            return ret;
        }

        public string[] GetRolesForUser(string username, string applicationName)
        {
            IQuery qr = Container.Query();
            qr.Constrain(typeof(db4oUserInRole));
            qr.Descend("_userName").Constrain(username);
            qr.Descend("_role").Descend("_application").Descend("_applicationName").Constrain(applicationName);
            IObjectSet result = qr.Execute();
            string[] ret = new string[result.Count];
            int i = 0;
            while (result.HasNext())
            {
                ret[i] = (((db4oUserInRole)result.Next()).Role.RoleName);
                i++;
            }
            return ret;
        }

        public void AddUserToRole(string username, string roleName, string applicationName)
        {
            db4oRole role = GetRoleByName(roleName, applicationName);
            if (role != null)
            {
                if (Membership.GetUser(username) != null)
                {
                    if (IsUserInRole(username, roleName, applicationName))
                        throw new ProviderException("The user " + username + " is already in role " + roleName);
                    db4oUserInRole uir = new db4oUserInRole();
                    uir.Role = role;
                    uir.UserName = username;
                    Container.Set(uir);
                }
                else
                {
                    throw new ArgumentException("User " + username + " not found");
                }
            }
            else
                throw new ArgumentException("Role " + roleName + " not found");

        }

        public bool IsUserInRole(string userName, string roleName, string applicationName)
        {
            return GetUserInRoleObject(userName, roleName, applicationName) != null;
        }

        public void RemoveUserToRole(string userName, string roleName, string applicationName)
        {
            db4oUserInRole uir = GetUserInRoleObject(userName, roleName, applicationName);
            if (uir != null)
            {
                Container.Delete(uir);
                Container.Commit();
            }
            else
                throw new ProviderException("User " + userName + " not found in rule " + roleName);

        }

        private db4oUserInRole GetUserInRoleObject(string username, string rolename,string applicationName)
        {
            IQuery qr = Container.Query();
            qr.Constrain(typeof(db4oUserInRole));
            qr.Descend("_userName").Constrain(username);
            qr.Descend("_role").Descend("_rolename").Constrain(rolename);
            qr.Descend("_role").Descend("_application").Descend("_applicationName").Constrain(applicationName);
            if (qr.Execute().HasNext())
                return (db4oUserInRole)(qr.Execute().Next());
            else
                return null;
        }
    }
}
