using System;
using System.Collections.Specialized;
using System.Text;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Configuration.Provider;
using System.Text.RegularExpressions;
using System.Security.Cryptography;

namespace Deerchao.Db4oWebSupport.Security
{
    public class db4oMembershipProvider : MembershipProvider
    {
        private const int saltSize = 16;

        # region Fields
        private string _connString = string.Empty;
        private bool _enablePasswordRetrieval;
        private bool _enablePasswordReset;
        private bool _requiresQuestionAndAnswer;
        private string _applicationName;
        
        private bool _requiresUniqueEmail;       
        private string _hashAlgorithmType;       
        private int _maxInvalidPasswordAttempts;
        private int _passwordAttemptWindow;
        private int _minRequiredPasswordLength;
        private int _minRequiredNonAlphanumericCharacters;
        private string _passwordStrengthRegularExpression;        
        private MembershipPasswordFormat _PasswordFormat;
		private string _connStringName;

		private db4oMembershipProviderManager _manager;

        #endregion		

		public db4oMembershipProviderManager Manager
		{
			get
			{
				if (_manager == null)
					_manager = db4oMembershipProviderManager.GetManager(this._connStringName);
				return _manager;
			}
		}

        #region Properties

       
		
        public override int MaxInvalidPasswordAttempts
        {
            get
            {                
                return this._maxInvalidPasswordAttempts;
            }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get
            {
                return this._minRequiredNonAlphanumericCharacters;
            }
        }

        public override int MinRequiredPasswordLength
        {
            get
            {
                return this._minRequiredPasswordLength;
            }
        }

        public override int PasswordAttemptWindow
        {
            get
            {
                return this._passwordAttemptWindow;
            }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get
            {
                return this._PasswordFormat;
            }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get 
            {
                return _requiresQuestionAndAnswer;
            }
        }

        public override bool RequiresUniqueEmail
        {
			get { return _requiresUniqueEmail; }
        }

        public override string ResetPassword(string username, string answer)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override bool UnlockUser(string userName)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override void UpdateUser(MembershipUser user)
        {
			db4oMembershipUser template = new db4oMembershipUser();
			template.Id = user.ProviderUserKey.ToString();
			template.Comment = user.Comment;
			template.Email = user.Email;
			template.IsApproved = user.IsApproved;
			template.IsLoockedOut = user.IsLockedOut;												
			template.PasswordQuestion = user.PasswordQuestion;
			template.UserName = user.UserName;
			Manager.UpdateUser(template);
        }

        public override bool ValidateUser(string username, string password)
        {
			if ( !ValidatePassword(password) )
				return false;
			db4oMembershipUser user = Manager.GetUserByUserName(username);
			if(user != null && ValidadeUserPassword(user,password))
			{
				if ( !user.IsLoockedOut & user.IsApproved )
				{
					Manager.RegisterActivity(username, this.ApplicationName,ActivityType.Logging);
					return true;
				}
				else
				{
					//TODO: throw a exception?
					return false;
				}

			}
			else
			{
				return false;
			}
			
        }

		private bool ValidadeUserPassword(db4oMembershipUser user, string password)
		{
			string  encSupplyPassword = EncodePassword(password, user.PasswordFormat, user.PasswordSalt);
			if ( encSupplyPassword == user.Password )
				return true;
			else
				return false;
		}

        public override bool EnablePasswordReset
        {
            get
            {
                return this._enablePasswordReset;
            }
        }

        public override bool EnablePasswordRetrieval
        {
            get
            {
                return this._enablePasswordRetrieval;
            }
        }

        public override string ApplicationName
        {
            get
            {
                return this._applicationName;
            }
            set
            {
                this._applicationName = value;
            }
        }

        #endregion

        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            if (config == null)
                throw new ArgumentNullException("config");

            if (String.IsNullOrEmpty(name))
                name = "db4oMembershipProvider";

            if (string.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "Membership ASP.NET db4o Provider");
            }

            base.Initialize(name, config);

            this._applicationName = config["applicationName"];
            this._connStringName = config["connectionStringName"].ToString();
            this._enablePasswordRetrieval = Utils.ReadBooleanValue(config, "enablePasswordRetrieval", false);
            this._enablePasswordReset = Utils.ReadBooleanValue(config, "enablePasswordReset", true);
            this._requiresQuestionAndAnswer = Utils.ReadBooleanValue(config, "requiresQuestionAndAnswer", true);
            this._requiresUniqueEmail = Utils.ReadBooleanValue(config, "requiresUniqueEmail", true);
            

            _passwordStrengthRegularExpression = config["passwordStrengthRegularExpression"];
            if (_passwordStrengthRegularExpression != null)
            {
                _passwordStrengthRegularExpression = _passwordStrengthRegularExpression.Trim();
                if (_passwordStrengthRegularExpression.Length != 0)
                {
                    try
                    {
                        Regex regex = new Regex(_passwordStrengthRegularExpression);
                    }
                    catch (ArgumentException e)
                    {
                        throw new ProviderException(e.Message, e);
                    }
                }
            }
            else
            {
                _passwordStrengthRegularExpression = string.Empty;
            }

            this._maxInvalidPasswordAttempts = Utils.ReadIntegerValue(config, "maxInvalidPasswordAttempts", 5, false, 0);
            this._passwordAttemptWindow = Utils.ReadIntegerValue(config, "passwordAttemptWindow", 10, false, 0);
            this._minRequiredPasswordLength = Utils.ReadIntegerValue(config, "minRequiredPasswordLength", 7, false, 128);
            this._minRequiredNonAlphanumericCharacters = Utils.ReadIntegerValue(config, "minRequiredNonalphanumericCharacters", 1, true, 128);
            
            _hashAlgorithmType = config["hashAlgorithmType"];
            if (String.IsNullOrEmpty(_hashAlgorithmType))
            {
                _hashAlgorithmType = "SHA1";
            }

            string passformat = config["passwordFormat"];
            if (passformat == null)
                passformat = "Hashed";

            switch (passformat)
            {
                case "Clear":
                    _PasswordFormat = MembershipPasswordFormat.Clear;
                    break;
                case "Encrypted":
                    _PasswordFormat = MembershipPasswordFormat.Encrypted;
                    break;
                case "Hashed":
                    _PasswordFormat = MembershipPasswordFormat.Hashed;
                    break;
                default:
                    throw new ProviderException("Unkown password format");
            }



        }
		
        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
			if ( ValidatePassword(oldPassword) && ValidatePassword(newPassword) )
			{
				db4oMembershipUser user = Manager.GetUserByUserName(username);
				if ( user != null )
				{
					if ( ValidadeUserPassword(user, oldPassword) )
					{
						string salt = GenerateSalt();
						string encPass = EncodePassword(newPassword, (int)this._PasswordFormat, salt);
						if ( Manager.ChangePassword(user.UserName, encPass, salt, _PasswordFormat) )
						{
							Manager.RegisterActivity(user.UserName, this.ApplicationName, ActivityType.ChangePassword);
							return true;
						}
						else
							throw new MembershipPasswordException("Can't change password");
					}
					else
					{
						throw new MembershipPasswordException("Invalid password");
					}
				}
				else
				{
					throw new ProviderException("User not found");
				}			
			}
			else
			{
				throw new MembershipPasswordException("Invalid old and/or new password");
			}

        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
        {
			db4oMembershipUser user = Manager.GetUserByUserName(username);
			if ( user != null )
			{
				if ( ValidadeUserPassword(user, password) )
				{
					if ( Manager.ChangePasswordQuestionAndAnswer(user.UserName, newPasswordQuestion, newPasswordAnswer) )
					{
						Manager.RegisterActivity(user.UserName, this.ApplicationName, ActivityType.ChangePasswordQ_A);
						return true;
					}
					else
					{
						throw new ProviderException("Can't change password answer and questio");
					}
				}
				else
					throw new MembershipPasswordException("Invalid password");
			}
			else
			{
				throw new ProviderException("User not found");
			}
        }

        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            if (!ValidatePassword(password))
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            string salt = GenerateSalt();
            string pass = EncodePassword(password, (int)_PasswordFormat, salt);      
            

            try
            {
                // 1 - Verify if userId exist
                if (Manager.GetUserByUserName(username) != null)
                {
                    status = MembershipCreateStatus.DuplicateUserName;
                    return null;
                }
                // 2 - Verify eventually duplicate email
				if (Manager.GetUserByEmail(email) != null)
				{
					status = MembershipCreateStatus.DuplicateEmail;
					return null;
				}								

				MembershipUser user  = Manager.CreateUser(username, email,PasswordFormat, passwordAnswer, isApproved, salt, pass,this.ApplicationName,this.Name);

				if (user != null)
				{
					status = MembershipCreateStatus.Success;
					return user;

				}
				else
				{
					status = MembershipCreateStatus.ProviderError;
					return null;
				}
				                
            }
            catch
            {
                status = MembershipCreateStatus.ProviderError;
                return null;
            }            

        }
               
        private string GenerateSalt()
        {
            byte[] buf = new byte[saltSize];
            (new RNGCryptoServiceProvider()).GetBytes(buf);
            return Convert.ToBase64String(buf);
        }

        private string EncodePassword(string pass, int passwordFormat, string salt)
        {
            if (passwordFormat == 0) // MembershipPasswordFormat.Clear
                return pass;

            byte[] bIn = Encoding.Unicode.GetBytes(pass);
            byte[] bSalt = Convert.FromBase64String(salt);
            byte[] bAll = new byte[bSalt.Length + bIn.Length];
            byte[] bRet = null;

            Buffer.BlockCopy(bSalt, 0, bAll, 0, bSalt.Length);
            Buffer.BlockCopy(bIn, 0, bAll, bSalt.Length, bIn.Length);
            if (passwordFormat == 1)
            { // MembershipPasswordFormat.Hashed
                HashAlgorithm s = HashAlgorithm.Create(this._hashAlgorithmType);

                // If the hash algorithm is null (and came from config), throw a config exception
                if (s == null)
                {
                    throw new ProviderException("Could not create a hash algorithm");
                }
                bRet = s.ComputeHash(bAll);
            }
            else
            {
                bRet = EncryptPassword(bAll);
            }

            return Convert.ToBase64String(bRet);
        }

        private bool ValidatePassword(string password)
        {
            if (password.Length < this._minRequiredPasswordLength)
                return false;
               // throw new ProviderException(string.Format("The password must have {0} or more characters", this._minRequiredPasswordLength));
            if (this._minRequiredNonAlphanumericCharacters > 0)
            {
                int count = 0;
                foreach (char c in password)
                {
                    if (!char.IsLetterOrDigit(c))
                        count++;
                }
                if (count < this._minRequiredNonAlphanumericCharacters)
                    return false;
                //    throw new ProviderException(string.Format("The password must have {0} non alphanumeric characters",this._minRequiredNonAlphanumericCharacters)); 
            }
            
            return true;
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
			return Manager.DeleteUser(username, deleteAllRelatedData);
        }      

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
			if ( pageIndex < 0 )
				throw new ArgumentException("PageIndex cannot be negative");
			if ( pageSize < 1 )
				throw new ArgumentException("PageSize must be positive");

			double lBound = (double)(pageIndex * pageSize);
			double uBound = (double)(lBound + pageSize - 1);
			MembershipUserCollection users = new MembershipUserCollection();
			totalRecords = 0;
			int i = 0;
			foreach ( db4oMembershipUser user in Manager.GetUsersByName(usernameToMatch) )
			{
				totalRecords++;
				if ( i >= lBound & i <= uBound )
				{
					users.Add(Manager.GetMembershipByNames(user.UserName, this.ApplicationName, this.Name));
				}
			}
			return users;		
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
			if ( pageIndex < 0 )
				throw new ArgumentException("PageIndex cannot be negative");
			if ( pageSize < 1 )
				throw new ArgumentException("PageSize must be positive");

			double lBound = (double)(pageIndex * pageSize);
			double uBound = (double)(lBound + pageSize - 1);
			MembershipUserCollection users = new MembershipUserCollection();
			totalRecords = 0;
			int i = 0;
			foreach ( db4oMembershipUser user in Manager.GetAllUsers() )
			{
				totalRecords++;
				if ( i >= lBound & i <= uBound )
				{
					users.Add(Manager.GetMembershipByNames(user.UserName, this.ApplicationName, this.Name));
				}
			}
			return users;
        }

        public override int GetNumberOfUsersOnline()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override string GetPassword(string username, string answer)
        {
            if (!_enablePasswordRetrieval)				
                throw new ProviderException("Retrieve password not enabled");

            if (_PasswordFormat == MembershipPasswordFormat.Hashed && _enablePasswordRetrieval)
                throw new ProviderException("Provider cannot retrieve hashed password");

			db4oMembershipUser user = Manager.GetUserByUserName(username);
			if ( user != null )
			{
				if ( this.RequiresQuestionAndAnswer )
				{
					if ( string.Compare(user.PasswordAnswer, answer, true, System.Globalization.CultureInfo.InvariantCulture) == 0 )
						throw new MembershipPasswordException("Invalid answer");
				}
				Manager.RegisterActivity(username, this.ApplicationName, ActivityType.RequestLostPassword);
				return DecodePassword(user.Password, user.PasswordFormat);
			}
			else
			{
				throw new ProviderException("User not found!");
			}

                        
        }

		private string DecodePassword(string password, int passwordFormat)
		{
			if (passwordFormat == 0)
				return password;
			else
				if ( passwordFormat== 1 )
					throw new ProviderException("Provider cannot retrieve hashed password");
				else
				{
					byte[] bIn = Convert.FromBase64String(password);
					byte[] bRet = DecryptPassword(bIn);
					if ( bRet == null )
						return null;
					return Encoding.Unicode.GetString(bRet, saltSize, bRet.Length - saltSize);
				}
		}

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
			return Manager.GetMembershipByNames(username,this.ApplicationName,this.Name);
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
			return Manager.GetMembershipByUserId(providerUserKey.ToString(), this.ApplicationName, this.Name);
        }

        public override string GetUserNameByEmail(string email)
        {
			return Manager.GetUserByEmail(email).UserName;
        }
    }
}
