using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Security;
using System.Configuration.Provider;

namespace Deerchao.Db4oWebSupport.Security
{
    class db4oRoleProvider : RoleProvider
    {
        private string _applicationName;
        private string _connStringName;
        private db4oRoleProviderManager _manager;

        public db4oRoleProviderManager Manager
        {
            get
            {
                if (_manager == null)
                    _manager = db4oRoleProviderManager.GetManager(this._connStringName);
                return _manager;
            }
        }

        public override string ApplicationName
        {
            get
            {
                return _applicationName;
            }
            set
            {
                this._applicationName = value;
            }
        }

        public override string Description
        {
            get
            {
                return base.Description;
            }
        }

        public override string Name
        {
            get
            {
                return base.Name;
            }
        }

        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            if (config == null)
                throw new ArgumentNullException("config");

            if (String.IsNullOrEmpty(name))
                name = "db4oRoleProvider";

            if (string.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "db4o Role Provider");
            }

            base.Initialize(name, config);

            this._applicationName = config["applicationName"];
            this._connStringName = config["connectionStringName"].ToString();
        }

        public override void CreateRole(string roleName)
        {
            if (string.IsNullOrEmpty(roleName))
                throw new ArgumentException("rolename");
            if (Manager.GetRoleByName(roleName, this.ApplicationName) != null)
                throw new ProviderException("Provider role already exists: " + roleName);
            if (Manager.CreateRole(roleName, ApplicationName) == null)
                throw new ProviderException("Unknown provider failure");
        }

        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            if (throwOnPopulatedRole)
            {
                string[] users = GetUsersInRole(roleName);
                if (users != null && users.Length > 0)
                    throw new ProviderException("Role is not empty");
            }

            return Manager.DeleteRole(roleName, ApplicationName);
        }

        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            foreach (string rolename in roleNames)
            {
                foreach (string username in usernames)
                {
                    Manager.AddUserToRole(username, rolename, this.ApplicationName);
                }
            }
        }

        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            string[] usersInRole = Manager.GetUsersInRole(roleName, this.ApplicationName);
            List<string> result = new List<string>(usersInRole.Length);
            foreach (string user in usersInRole)
                if (user.Contains(usernameToMatch))
                    result.Add(user);
            return result.ToArray();
        }

        public override string[] GetAllRoles()
        {
            return Manager.GetAllRoles(this.ApplicationName);
        }

        public override string[] GetRolesForUser(string username)
        {
            return Manager.GetRolesForUser(username, ApplicationName);
        }

        public override string[] GetUsersInRole(string roleName)
        {
            return Manager.GetUsersInRole(roleName, ApplicationName);
        }

        public override bool IsUserInRole(string username, string roleName)
        {
            return Manager.IsUserInRole(username, roleName, ApplicationName);
        }

        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            foreach (string rolename in roleNames)
            {
                foreach (string username in usernames)
                {
                    Manager.RemoveUserToRole(username, rolename, this.ApplicationName);
                }
            }
        }

        public override bool RoleExists(string roleName)
        {
            return Manager.GetRoleByName(roleName, this.ApplicationName) != null;
        }
    }
}
