using System;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Configuration.Provider;
using System.Text;
using System.Web.Hosting;
using System.Globalization;

namespace System.Web.Security
{
    internal static class Utils
    {
        internal static bool ReadBooleanValue(NameValueCollection config, string propertyName, bool defaultValue)
        {
            string stringValue = config[propertyName];
            if (stringValue == null)
            {
                return defaultValue;
            }

            if (stringValue.ToLower().Equals("true"))
            {
                return true;
            }

            if (stringValue.ToLower().Equals("false"))
            {
                return false;
            }

            throw new Exception("Invalid boolean value for property '" + propertyName + "'");
        }

        internal static int ReadIntegerValue(NameValueCollection config, string propertyName, int defaultValue, bool zeroAllowed, int maxValueAllowed)
        {
            string stringValue = config[propertyName];

            if (stringValue == null)
            {
                return defaultValue;
            }

            int iValue;
            try
            {
                iValue = Convert.ToInt32(stringValue, CultureInfo.InvariantCulture);
            }
            catch (InvalidCastException e)
            {
                if (zeroAllowed)
                {
                    throw new Exception("The value must be a positive integer for property '" + propertyName + "'", e);
                }

                throw new Exception("The value must be a positive integer for property '" + propertyName + "'", e);
            }

            if (zeroAllowed && iValue < 0)
            {
                throw new Exception("The value must be a non-negative integer for property '" + propertyName + "'");
            }

            if (!zeroAllowed && iValue <= 0)
            {
                throw new Exception("The value must be a non-negative integer for property '" + propertyName + "'");
            }

            if (maxValueAllowed > 0 && iValue > maxValueAllowed)
            {
                throw new Exception("The value is too big for '" + propertyName + "' must be smaller than " + maxValueAllowed.ToString(CultureInfo.InvariantCulture));
            }

            return iValue;
        }

        internal static string GetDefaultApplicationName()
        {
            try
            {
                string applicationName = System.Web.HttpRuntime.AppDomainAppVirtualPath;
                if (applicationName == null || applicationName.Length == 0)
                {
                    return "/";
                }
                else
                {
                    return applicationName;
                }
            }
            catch
            {
                return "/";
            }
        }
    }
}
